import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import * as R from "ramda";
import counterModel from "Examples/ui-kit/Counter/model";
import searchBarModel from "Examples/ui-kit/SearchBar/model";
import { addReducer, addBridge, merge, addLifecycle } from "./decorators";
import { GLOBAL_UPDATE, DEFAULT_INSTANCE_ID, GLOBAL_CONTEXT_ID } from "./constants";
describe("## Redux-MVC decorators", function () {
  describe("### addReducer", function () {
    it("Should create the reducer", function () {
      var module = addReducer()(counterModel);
      expect(_typeof(module.reducer)).toBe("function");
    });
    it("Should return the ini state", function () {
      var module = addReducer()(counterModel);
      var iniState = module.reducer(undefined, {});
      expect(iniState).toBe(module.iniState);
    });
    it("Should execute the right reducer", function () {
      var module = addReducer()(counterModel);
      var nextState = module.reducer(module.iniState, module.actions.setCount(1));
      expect(nextState[module.namespace][DEFAULT_INSTANCE_ID]).toEqual({
        count: 1
      });
    });
    it("Should execute the right reducer and instance", function () {
      var module = addReducer()(counterModel);
      var nextState = module.reducer(module.iniState, module.actions.setCount(1, {
        meta: {
          instanceId: "counter"
        }
      }));
      expect(nextState[module.namespace].counter).toEqual({
        count: 1
      });
    });
    it("Should return the previous state if the `action.type` handler is not found", function () {
      var module = addReducer()(counterModel);
      var prevState = {};
      var nextState = module.reducer(prevState, {
        type: "fake"
      });
      expect(nextState).toBe(prevState);
    });
    it("Should ignore the instanceId if the model is a singleton", function () {
      var module = addReducer()(searchBarModel);
      var nextState = module.reducer(module.iniState, searchBarModel.actions.setSearch("hola", {
        meta: {
          instanceId: "searchBar"
        }
      }));
      expect(nextState[module.namespace][DEFAULT_INSTANCE_ID]).toEqual({
        search: "hola"
      });
    });
    it("Should merge the global updated state", function () {
      var module = addReducer()(searchBarModel);
      var nextState = module.reducer(module.iniState, {
        type: "action",
        payload: {
          newUser: "user"
        },
        meta: _defineProperty({}, GLOBAL_UPDATE, {
          user: "user"
        })
      });
      expect(nextState.user).toBe("user");
    });
    it("Should return the old state instance if the reducer is the identity", function () {
      var module = addReducer()(searchBarModel);
      var nextState = module.reducer(module.iniState, module.actions.identity());
      expect(nextState).toEqual(module.iniState);
    });
  });
  describe("### addBridge", function () {
    it("Should track all `module.dependencies` trackGlobalNamespaces and append the `trackGlobalNamespaces` param", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            trackGlobalNamespaces: ["App", "Routing"]
          },
          miniItemsSelector: {
            trackGlobalNamespaces: ["Event", "Routing"]
          }
        },
        namespace: "FakeModel"
      };
      var module = addBridge({
        trackGlobalNamespaces: ["App", "User"]
      })(mockModel);
      expect(module.trackGlobalNamespaces.length).toEqual(4);
      expect(module.trackGlobalNamespaces).toEqual(expect.arrayContaining(["App", "User", "Routing", "Event"]));
    });
    it("Should track all `module.dependencies` trackGlobalNamespaces", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            trackGlobalNamespaces: ["App", "Routing"]
          },
          miniItemsSelector: {
            trackGlobalNamespaces: ["Event", "Routing"]
          }
        },
        namespace: "FakeModel"
      };
      var module = addBridge()(mockModel);
      expect(module.trackGlobalNamespaces.length).toEqual(3);
      expect(module.trackGlobalNamespaces).toEqual(expect.arrayContaining(["App", "Routing", "Event"]));
    });
    it("Should set the `dispatchToGlobal` function", function () {
      var dispatchToGlobal = function dispatchToGlobal() {};

      var module = addBridge({
        dispatchToGlobal: dispatchToGlobal
      })(searchBarModel);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: dispatchToGlobal
      }));
    });
    it("Should set the default `dispatchToGlobal` function when `dispatchToGlobal` param is empty", function () {
      var mockModel = {
        dependencies: {
          searchBar: {
            namespace: "SearchBar"
          },
          miniItemsSelector: {
            namespace: "MiniItemsSelector"
          }
        },
        namespace: "FakeModel"
      };
      var module = addBridge()(mockModel);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: expect.any(Function)
      }));
      var defaultDispatchToGlobal = module.dispatchToGlobal;
      expect(defaultDispatchToGlobal({
        type: "SearchBar/create"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "MiniItemsSelector/setName"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "FakeModel/init"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "App/commit"
      })).toEqual(true);
    });
    it("Should set the default `dispatchToGlobal` even if module dependencies is empty", function () {
      var mockModel = {
        namespace: "FakeModel"
      };
      var module = addBridge()(mockModel);
      expect(module).toEqual(expect.objectContaining({
        dispatchToGlobal: expect.any(Function)
      }));
      var defaultDispatchToGlobal = module.dispatchToGlobal;
      expect(defaultDispatchToGlobal({
        type: "FakeModel/init"
      })).toEqual(false);
      expect(defaultDispatchToGlobal({
        type: "App/commit"
      })).toEqual(true);
    });
  });
  describe("### merge", function () {
    it("Should merge two modules", function () {
      var left = {
        namespace: "left",
        actions: "left",
        getters: "left",
        reducers: {
          left: ""
        },
        iniState: {
          left: ""
        },
        reducer: "left",
        dependencies: {
          Counter: counterModel
        },
        constructor: "left",
        componentDidCatch: "left",
        componentWillUnmount: "left",
        singleton: true,
        other: "left"
      };
      var right = {
        reducers: {
          right: ""
        },
        iniState: {
          right: ""
        },
        dependencies: {
          SearchBar: searchBarModel
        }
      };
      var module = merge(left)(right);
      expect(module).toEqual(expect.objectContaining(_objectSpread({}, right, {
        dependencies: _objectSpread({}, left.dependencies, {}, right.dependencies, _defineProperty({}, left.namespace, left))
      })));
      expect(module).toEqual(expect.not.objectContaining(left));
    });
    it("Should merge two modules with no dependencies", function () {
      var module = merge(searchBarModel)(counterModel);
      expect(module).toEqual(expect.objectContaining(_objectSpread({}, counterModel, {
        dependencies: _defineProperty({}, searchBarModel.namespace, searchBarModel)
      })));
    });
  });
  describe("### addLifecycle", function () {
    it("Should add `constructor` and `componentWillUnmount` to the module", function () {
      var newModule = R.compose(addLifecycle(), addReducer())(counterModel);
      expect(newModule).toEqual(expect.objectContaining({
        constructor: expect.any(Function),
        componentWillUnmount: expect.any(Function)
      }));
    });
    it("Should create the store", function () {
      var newModule = R.compose(addLifecycle(), addReducer())(counterModel);
      var contextId = "fakeContextId";

      var moduleInstances = _defineProperty({}, contextId, _objectSpread({}, newModule));

      var moduleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      expect(moduleInstance.store).toEqual(expect.objectContaining({
        getState: expect.any(Function),
        subscribe: expect.any(Function),
        dispatch: expect.any(Function)
      }));
      moduleInstance.store.dispatch(newModule.actions.add());
      expect(newModule.getters.count(moduleInstance.store.getState())).toEqual(1);
      moduleInstance.store.dispatch(newModule.actions.add(null, {
        meta: {
          instanceId: "fakeInstance"
        }
      }));
      expect(newModule.getters.count(moduleInstance.store.getState(), {
        instanceId: "fakeInstance"
      })).toEqual(1);
    });
    it("Should persist the store by default", function () {
      var newModule = R.compose(addLifecycle(), addReducer())(counterModel);
      var contextId = "fakeContextId";

      var moduleInstances = _defineProperty({}, contextId, _objectSpread({}, newModule));

      var moduleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      var newModuleInstance = newModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      expect(moduleInstance.store.getState()).toEqual(newModuleInstance.store.getState());
    });
    it("Should dispatch to the global instance and track the result", function () {
      var _moduleInstances3;

      var searchBarModule = R.compose(addLifecycle(), addReducer())(searchBarModel);
      var counterModule = R.compose(addLifecycle(), addReducer(), addBridge({
        trackGlobalNamespaces: [searchBarModule.namespace]
      }))(counterModel);
      var contextId = "fakeContextId";
      var moduleInstances = (_moduleInstances3 = {}, _defineProperty(_moduleInstances3, contextId, _objectSpread({}, counterModule)), _defineProperty(_moduleInstances3, GLOBAL_CONTEXT_ID, _objectSpread({}, searchBarModule)), _moduleInstances3);
      var globalInstance = searchBarModule.constructor({
        moduleInstances: moduleInstances,
        contextId: GLOBAL_CONTEXT_ID
      });
      moduleInstances[GLOBAL_CONTEXT_ID] = globalInstance;
      var moduleInstance = counterModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("hola"));
      expect(searchBarModule.getters.search(globalInstance.store.getState())).toEqual("hola");
      expect(searchBarModule.getters.search(moduleInstance.store.getState())).toEqual("hola");
    });
    it("Should unbind the bridge on `componentWillUnmount`", function () {
      var _moduleInstances4;

      var searchBarModule = R.compose(addLifecycle(), addReducer())(searchBarModel);
      var counterModule = R.compose(addLifecycle(), addReducer(), addBridge({
        trackGlobalNamespaces: [searchBarModule.namespace]
      }))(counterModel);
      var contextId = "fakeContextId";
      var moduleInstances = (_moduleInstances4 = {}, _defineProperty(_moduleInstances4, contextId, _objectSpread({}, counterModule)), _defineProperty(_moduleInstances4, GLOBAL_CONTEXT_ID, _objectSpread({}, searchBarModule)), _moduleInstances4);
      var globalInstance = searchBarModule.constructor({
        moduleInstances: moduleInstances,
        contextId: GLOBAL_CONTEXT_ID
      });
      moduleInstances[GLOBAL_CONTEXT_ID] = globalInstance;
      var moduleInstance = counterModule.constructor({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstances[contextId] = moduleInstance;
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("hola"));
      counterModule.componentWillUnmount({
        moduleInstances: moduleInstances,
        contextId: contextId
      });
      moduleInstance.store.dispatch(searchBarModule.actions.setSearch("chau"));
      expect(searchBarModule.getters.search(globalInstance.store.getState())).toEqual("hola");
      expect(searchBarModule.getters.search(moduleInstance.store.getState())).toEqual("hola");
    });
  });
});