import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import * as R from "ramda";
import counterModule from "Examples/ui-kit/Counter";
import searchBarModule from "Examples/ui-kit/SearchBar";
import { createSelector } from "./selectors";
import { merge, addReducer } from "./decorators";
describe("## Create Selector", function () {
  it("Should return a selector function", function () {
    var selector = createSelector(counterModule.getters.count, R.inc);
    expect(selector).toEqual(expect.any(Function));
    expect(_typeof(selector.id)).toEqual("symbol");
    expect(selector.dependencies).toEqual([counterModule.getters.count]);
    expect(selector.resultFn).toEqual(R.inc);
    expect(selector.ownCache).toEqual(expect.objectContaining(_defineProperty({}, selector.id, {})));
  });
  it("Should return the same function if no dependencies where passed", function () {
    var selector = createSelector(counterModule.getters.count);
    expect(selector).toEqual(counterModule.getters.count);
  });
  it("Should compute the value once if dependencies dont change", function () {
    var count = jest.fn(counterModule.getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = createSelector(count, resultFn);
    var value = selector(counterModule.iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = selector(counterModule.iniState);
    expect(newValue).toEqual(1);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(1);
  });
  it("Should re-compute the value if dependencies change", function () {
    var count = jest.fn(counterModule.getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = createSelector(count, resultFn);
    var value = selector(counterModule.iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = selector(counterModule.reducer(counterModule.iniState, counterModule.actions.add()));
    expect(newValue).toEqual(2);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should re-compute the value for a different instanceId", function () {
    var count = jest.fn(counterModule.getters.count);
    var resultFn = jest.fn(R.inc);
    var selector = createSelector(count, resultFn);
    var value = selector(counterModule.iniState);
    expect(value).toEqual(1);
    expect(count).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var instanceValue = selector(counterModule.reducer(counterModule.iniState, counterModule.actions.add()), {
      instanceId: "counter"
    });
    expect(instanceValue).toEqual(1);
    expect(count).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should compose selectors and call the dependencies and result functions when needed", function () {
    var module = addReducer()(merge(searchBarModule)(counterModule));
    var count = jest.fn(counterModule.getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = createSelector(count, inc);
    var search = jest.fn(searchBarModule.getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = createSelector(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = createSelector(greetings, oneMore, resultFn);
    var value = onePlusGreetings(module.iniState);
    expect(value).toEqual("el 1");
    expect(count).toHaveBeenCalledTimes(1);
    expect(inc).toHaveBeenCalledTimes(1);
    expect(search).toHaveBeenCalledTimes(1);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newValue = onePlusGreetings(module.reducer(module.iniState, counterModule.actions.add()));
    expect(newValue).toEqual("el 2");
    expect(count).toHaveBeenCalledTimes(2);
    expect(inc).toHaveBeenCalledTimes(2);
    expect(search).toHaveBeenCalledTimes(2);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
  it("Should use the cache parameter", function () {
    var _expect$objectContain2;

    var module = addReducer()(merge(searchBarModule)(counterModule));
    var count = jest.fn(counterModule.getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = createSelector(count, inc);
    var search = jest.fn(searchBarModule.getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = createSelector(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = createSelector(greetings, oneMore, resultFn);
    var cache = {};
    onePlusGreetings(module.iniState, {}, cache);
    expect(cache).toEqual(expect.objectContaining((_expect$objectContain2 = {}, _defineProperty(_expect$objectContain2, onePlusGreetings.id, expect.any(Object)), _defineProperty(_expect$objectContain2, greetings.id, expect.any(Object)), _defineProperty(_expect$objectContain2, oneMore.id, expect.any(Object)), _expect$objectContain2)));
  });
  it("Should recompute the resultFn with a new cache", function () {
    var module = addReducer()(merge(searchBarModule)(counterModule));
    var count = jest.fn(counterModule.getters.count);
    var inc = jest.fn(R.compose(R.toString, R.inc));
    var oneMore = createSelector(count, inc);
    var search = jest.fn(searchBarModule.getters.search);
    var prepend = jest.fn(R.concat("el "));
    var greetings = createSelector(search, prepend);
    var resultFn = jest.fn(R.concat);
    var onePlusGreetings = createSelector(greetings, oneMore, resultFn);
    var cache = {};
    onePlusGreetings(module.iniState, {}, cache);
    expect(count).toHaveBeenCalledTimes(1);
    expect(inc).toHaveBeenCalledTimes(1);
    expect(search).toHaveBeenCalledTimes(1);
    expect(prepend).toHaveBeenCalledTimes(1);
    expect(resultFn).toHaveBeenCalledTimes(1);
    var newCache = {};
    onePlusGreetings(module.iniState, {}, newCache);
    expect(count).toHaveBeenCalledTimes(2);
    expect(inc).toHaveBeenCalledTimes(2);
    expect(search).toHaveBeenCalledTimes(2);
    expect(prepend).toHaveBeenCalledTimes(2);
    expect(resultFn).toHaveBeenCalledTimes(2);
  });
});